package at.ac.uibk.gitsearch.web.rest;

import at.ac.uibk.gitsearch.service.WatchListEntryService;
import at.ac.uibk.gitsearch.web.rest.errors.BadRequestAlertException;
import at.ac.uibk.gitsearch.service.dto.WatchListEntryDTO;

import io.github.jhipster.web.util.HeaderUtil;
import io.github.jhipster.web.util.PaginationUtil;
import io.github.jhipster.web.util.ResponseUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.web.servlet.support.ServletUriComponentsBuilder;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Optional;
import java.util.stream.StreamSupport;

import static org.elasticsearch.index.query.QueryBuilders.*;

/**
 * REST controller for managing {@link at.ac.uibk.gitsearch.domain.WatchListEntry}.
 */
@RestController
@RequestMapping("/api")
public class WatchListEntryResource {

    private final Logger log = LoggerFactory.getLogger(WatchListEntryResource.class);

    private static final String ENTITY_NAME = "watchListEntry";

    @Value("${jhipster.clientApp.name}")
    private String applicationName;

    private final WatchListEntryService watchListEntryService;

    public WatchListEntryResource(WatchListEntryService watchListEntryService) {
        this.watchListEntryService = watchListEntryService;
    }

    /**
     * {@code POST  /watch-list-entries} : Create a new watchListEntry.
     *
     * @param watchListEntryDTO the watchListEntryDTO to create.
     * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with body the new watchListEntryDTO, or with status {@code 400 (Bad Request)} if the watchListEntry has already an ID.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PostMapping("/watch-list-entries")
    public ResponseEntity<WatchListEntryDTO> createWatchListEntry(@Valid @RequestBody WatchListEntryDTO watchListEntryDTO) throws URISyntaxException {
        log.debug("REST request to save WatchListEntry : {}", watchListEntryDTO);
        if (watchListEntryDTO.getId() != null) {
            throw new BadRequestAlertException("A new watchListEntry cannot already have an ID", ENTITY_NAME, "idexists");
        }
        WatchListEntryDTO result = watchListEntryService.save(watchListEntryDTO);
        return ResponseEntity.created(new URI("/api/watch-list-entries/" + result.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, result.getId().toString()))
            .body(result);
    }

    /**
     * {@code PUT  /watch-list-entries} : Updates an existing watchListEntry.
     *
     * @param watchListEntryDTO the watchListEntryDTO to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated watchListEntryDTO,
     * or with status {@code 400 (Bad Request)} if the watchListEntryDTO is not valid,
     * or with status {@code 500 (Internal Server Error)} if the watchListEntryDTO couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PutMapping("/watch-list-entries")
    public ResponseEntity<WatchListEntryDTO> updateWatchListEntry(@Valid @RequestBody WatchListEntryDTO watchListEntryDTO) throws URISyntaxException {
        log.debug("REST request to update WatchListEntry : {}", watchListEntryDTO);
        if (watchListEntryDTO.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        WatchListEntryDTO result = watchListEntryService.save(watchListEntryDTO);
        return ResponseEntity.ok()
            .headers(HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, watchListEntryDTO.getId().toString()))
            .body(result);
    }

    /**
     * {@code GET  /watch-list-entries} : get all the watchListEntries.
     *
     * @param pageable the pagination information.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the list of watchListEntries in body.
     */
    @GetMapping("/watch-list-entries")
    public ResponseEntity<List<WatchListEntryDTO>> getAllWatchListEntries(Pageable pageable) {
        log.debug("REST request to get a page of WatchListEntries");
        Page<WatchListEntryDTO> page = watchListEntryService.findAll(pageable);
        HttpHeaders headers = PaginationUtil.generatePaginationHttpHeaders(ServletUriComponentsBuilder.fromCurrentRequest(), page);
        return ResponseEntity.ok().headers(headers).body(page.getContent());
    }

    /**
     * {@code GET  /watch-list-entries/:id} : get the "id" watchListEntry.
     *
     * @param id the id of the watchListEntryDTO to retrieve.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the watchListEntryDTO, or with status {@code 404 (Not Found)}.
     */
    @GetMapping("/watch-list-entries/{id}")
    public ResponseEntity<WatchListEntryDTO> getWatchListEntry(@PathVariable Long id) {
        log.debug("REST request to get WatchListEntry : {}", id);
        Optional<WatchListEntryDTO> watchListEntryDTO = watchListEntryService.findOne(id);
        return ResponseUtil.wrapOrNotFound(watchListEntryDTO);
    }

    /**
     * {@code DELETE  /watch-list-entries/:id} : delete the "id" watchListEntry.
     *
     * @param id the id of the watchListEntryDTO to delete.
     * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
     */
    @DeleteMapping("/watch-list-entries/{id}")
    public ResponseEntity<Void> deleteWatchListEntry(@PathVariable Long id) {
        log.debug("REST request to delete WatchListEntry : {}", id);
        watchListEntryService.delete(id);
        return ResponseEntity.noContent().headers(HeaderUtil.createEntityDeletionAlert(applicationName, true, ENTITY_NAME, id.toString())).build();
    }

    /**
     * {@code SEARCH  /_search/watch-list-entries?query=:query} : search for the watchListEntry corresponding
     * to the query.
     *
     * @param query the query of the watchListEntry search.
     * @param pageable the pagination information.
     * @return the result of the search.
     */
    @GetMapping("/_search/watch-list-entries")
    public ResponseEntity<List<WatchListEntryDTO>> searchWatchListEntries(@RequestParam String query, Pageable pageable) {
        log.debug("REST request to search for a page of WatchListEntries for query {}", query);
        Page<WatchListEntryDTO> page = watchListEntryService.search(0L);
        HttpHeaders headers = PaginationUtil.generatePaginationHttpHeaders(ServletUriComponentsBuilder.fromCurrentRequest(), page);
        return ResponseEntity.ok().headers(headers).body(page.getContent());
        }
}
